#!/usr/bin/python
#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/
# Author: Pat Marion
# Modified by Xiaoxiao Liu.

## This script is designed to run StriIncludes.py in parallel to significantly reduce
## the processing time for removing unnecessary header includes.

## To run the script you need to edit StripIncludes.py.
## You should also set up the number of processes according to the machine configuration.


from StripIncludes import *
from multiprocessing import Pool

###############  Inputs: need edit  #############
FILES_PER_PROCESS = 2
NUMBER_OF_PROCESSES = 8
#################################################

def main():

    fileList = open(relativeFileList, 'r').read().splitlines()

    args = []
    for i in xrange(0, len(fileList), FILES_PER_PROCESS):
        args.append(fileList[i:i+FILES_PER_PROCESS])

    pool = Pool(processes=NUMBER_OF_PROCESSES)
    pool.map(processFileList, args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import unicode literals so that StringIO works on both Python 2 and 3
from __future__ import unicode_literals
from __future__ import print_function

import sys
import os
import re
from argparse import ArgumentParser

try:
    # Python 3
    from io import StringIO
except ImportError:
    # Python 2
    from cStringIO import StringIO


def getType(v):
    if hasattr(v, "decl_type"):
        return getType(v.decl_type)
    if hasattr(v, "declaration"):
        return getType(v.declaration)
    return v


class IdxGenerator(object):
    """Generates a the .idx file for an ITK wrapping submodule (which usually
    corresponds to a class)."""

    def __init__(self, moduleName):
        self.moduleName = moduleName
        # the output file
        self.outputFile = StringIO()

    def create_idxfile(self, idxFilePath, wrappersNamespace):
        # iterate over all the typedefs in the _wrapping_::wrappers namespace
        for typedef in wrappersNamespace.typedefs():
            n = typedef.name
            s = getType(typedef).decl_string
            # drop the :: prefix - it make swig produce invalid code
            if s.startswith("::"):
                s = s[2:]
            self.outputFile.write("{%s} {%s} {%s}\n" % (s, n, self.moduleName))

        content = self.outputFile.getvalue()

        with open(idxFilePath, "w") as f:
            f.write(content)


class SwigInputGenerator(object):
    """Generates a swig input .i file for an ITK module."""

    notWrapped = [
        "std::_Deque_alloc<.+>",
        "itk::AtomicInt<.+>",
        "itk::MapContainer< unsigned long, itk::CellInterface<.+>",
        "itk::VectorContainer< unsigned long, itk::CellInterface<.+>",
        "itk::CellInterface< double, itk::QuadEdgeMeshCellTraitsInfo<.+>",
        "itk::QuadEdgeMeshLineCell< itk::CellInterface<.+>",
        "itk::SmartPointerForwardReference<.+>",
        "itk::LibHandle",
        "itk::NeighborhoodAllocator<.+>",
        # to avoid wrapping all the region for all the dims
        "itk::ImageRegion<.+>",
        "itk::ImportImageContainer<.+>",
        "itk::DefaultPixelAccessor<.+>",
        "itk::NeighborhoodAccessorFunctor<.+>",
        "itk::DefaultVectorPixelAccessor<.+>",
        "itk::VectorImageNeighborhoodAccessorFunctor<.+>",
        "itk::.*Iterator.*",  # TODO: remove this one ?
        "itk::Neighborhood<.+>",  # TODO: remove this one
        "itk::ThreadFunctionType",
        "itk::Functor::.+",
        "itk::SmartPointer< itk::Functor::.+",
        "itk::Function::.+",
        "itk::.+Function.*",  # Level set functions
        "itk::watershed::.+",  # ignore the internal classes of the watershed
        # require to wrap too more type
        "itk::SmartPointer< itk::VoronoiDiagram2D<.+> >",
        # used internally in ImageToImageMetric
        "itk::Image< itk::CovariantVector< double, \d+u >, \d+u >",
        "itk::FixedArray< itk::SmartPointer.+ >",
        # used internally in itkTransformBase
        "itk::SmartPointer< itk::Transform.+ >",
        # used internally in itkMattesMutualInformationImageToImageMetric
        "itk::SmartPointer< itk::Image.+ >",
        "itk::ObjectFactoryBasePrivate"
    ]

    notWrappedRegExp = re.compile("|".join(["^" + s + "$" for s in notWrapped]))

    # stdcomplex code

    stdcomplex_headers = {
        "D": """ class stdcomplexD {
       public:
         ~stdcomplexD();
         stdcomplexD & operator=(stdcomplexD const & arg0);
         stdcomplexD(stdcomplexD const & arg0);
         stdcomplexD(stdcomplexD __z);
         stdcomplexD(double __r = 0.0, double __i = 0.0);
         stdcomplexD(stdcomplexF const & __z);
         double real();
         double const real() const;
         double imag();
         double const imag() const;
         stdcomplexD & operator=(double __d);
         stdcomplexD & operator+=(double __d);
         stdcomplexD & operator-=(double __d);
         stdcomplexD & operator*=(double __d);
         stdcomplexD & operator/=(double __d);
         // stdcomplexD const & __rep() const;
       private:
       protected:
     };
    """,

        "F": """class stdcomplexF {
       public:
         ~stdcomplexF();
         stdcomplexF & operator=(stdcomplexF const & arg0);
         stdcomplexF(stdcomplexF const & arg0);
         stdcomplexF(stdcomplexF __z);
         stdcomplexF(float r = 0.0f, float i = 0.0f);
         stdcomplexF(stdcomplexD const & __z);
         float real();
         float const real() const;
         float imag();
         float const imag() const;
         stdcomplexF & operator=(float __f);
         stdcomplexF & operator+=(float __f);
         stdcomplexF & operator-=(float __f);
         stdcomplexF & operator*=(float __f);
         stdcomplexF & operator/=(float __f);
         // stdcomplexF const & __rep() const;
       private:
       protected:
     };
    """}

    def __init__(self, moduleName, options):
        self.moduleName = moduleName
        self.options = options

        self.outputFile = StringIO()
        self.applyFileNames = []

        # a dict to let us use the alias name instead of the full c++ name. Without
        # that, in many cases, swig don't know that's the same type
        self.aliases = {}

        # a set of used types
        self.usedTypes = set()

        # a dict to store the file where the def comes from
        self.typedefSource = {}

        self.warnings = set()

        self.mdx_loaded = set()

        self.verbose = options.verbose


    def warn(self, id, msg, doWarn=True):
        if not doWarn:
            # don't warn for anything
            return
        if str(id) not in self.options.warnings:
            if not self.verbose and (id, msg) in self.warnings:
                # just do nothing
                return
            self.warnings.add((id, msg))
            if self.verbose:
                if self.options.warningError:
                    print("error(%s): %s" % (str(id), msg), file=sys.stderr)
                else:
                    print("warning(%s): %s" % (str(id), msg), file=sys.stderr)
            else:
                if self.options.warningError:
                    print(
                        "%s: error(%s): %s" %
                        (self.moduleName, str(id), msg), file=sys.stderr)
                else:
                    print(
                        "%s: warning(%s): %s" %
                        (self.moduleName, str(id), msg), file=sys.stderr)

    def info(self, msg):
        if self.verbose:
            print("info: %s" % msg, file=sys.stderr)

    @staticmethod
    def getDeclarationString(t):
        t = getType(t)
        if t.decl_string == "::PyObject *":
            # don't go further - we want to keep that one as is
            return "::PyObject *"
        if isinstance(t, pygccxml.declarations.cpptypes.pointer_t):
            return SwigInputGenerator.getDeclarationString(getType(t.base)) + " *"
        elif isinstance(t, pygccxml.declarations.cpptypes.const_t):
            return SwigInputGenerator.getDeclarationString(getType(t.base)) + " const"
        elif isinstance(t, pygccxml.declarations.cpptypes.reference_t):
            return SwigInputGenerator.getDeclarationString(getType(t.base)) + " &"
        return t.decl_string

    def renameTypesInSTL(self, s):
        if s.startswith("std::") and \
                pygccxml.declarations.templates.is_instantiation(s):
            args = []
            for arg in pygccxml.declarations.templates.args(s):
                t, d = SwigInputGenerator.typeAndDecorators(arg)
                args.append(self.renameTypesInSTL(self.get_alias(t)) + d)
            return pygccxml.declarations.templates.join(
                pygccxml.declarations.templates.name(s),
                args) + SwigInputGenerator.typeAndDecorators(s)[1]
        return s

    @staticmethod
    def removeStdAllocator(s):
        if pygccxml.declarations.templates.is_instantiation(s):
            args = []
            for arg in pygccxml.declarations.templates.args(s):
                if not arg.startswith("std::allocator"):
                    t, d = SwigInputGenerator.typeAndDecorators(arg)
                    args.append(SwigInputGenerator.removeStdAllocator(t) + d)
            return pygccxml.declarations.templates.join(
                pygccxml.declarations.templates.name(s),
                args) + SwigInputGenerator.typeAndDecorators(s)[1]
        return s

    @staticmethod
    def typeAndDecorators(s):
        end = ""
        s = s.strip()
        ends = [" ", "*", "&", "const"]
        needToContinue = True
        while needToContinue:
            needToContinue = False
            for e in ends:
                if s.endswith(e):
                    end = e + end
                    s = s[:-len(e)]
                    needToContinue = True
        return (s, end)

    def get_alias(self, decl_string, w=True):
        s = str(decl_string)

        # drop the :: prefix - it make swig produce invalid code
        if s.startswith("::"):
            s = s[2:]

        # normalize string
        s = SwigInputGenerator.normalize(s)

        # workaround a bug - or is it a feature ? - somewhere
        s = s.replace("complex float", "std::complex<float>")
        s = s.replace("complex double", "std::complex<double>")
        s = s.replace("complex long double", "std::complex<long double>")

        (s, end) = SwigInputGenerator.typeAndDecorators(s)

        if s in self.aliases:
            self.usedTypes.add(self.aliases[s])
            return self.aliases[s] + end

        if s.startswith("itk::Templates::"):
            # that's a explicitly instantiated type. The name is the same than
            # the WrapITK one, so lets use it as a base for WrapITK
            # Ex: itk::Templates::RGBPixelUC
            # don't store the new string in s, because we need it unchanged if
            # the type is explicitly instantiated, but not wrapped
            new_s = s.replace("::Templates::", "")
            if new_s.split("::")[0] in self.aliases.values():
                self.usedTypes.add(new_s)
                return new_s + end

        if s[:s.rfind("::")] in self.aliases:
            # take care of subtypes/enum/...
            alias = self.aliases[s[:s.rfind("::")]] + s[s.rfind("::"):]
            self.usedTypes.add(alias)
            return alias + end

        # replace the types defined in this type, to support
        # std::vector<itkDataObject> for example
        s = self.renameTypesInSTL(s)

        # drop the allocator part of the type, because it is not supported by the
        # %template directive with some generators (like tcl)
        s = SwigInputGenerator.removeStdAllocator(s)

        # rename basic_string to std::string to make name shorter
        s = s.replace("std::basic_string< char >", "std::string")
        s = s.replace(
            "std::basic_string< char, std::char_traits< char > >",
            "std::string")
        s = s.replace(
            "std::basic_ostream< char, std::char_traits< char > >",
            "std::ostream")
        s = s.replace(
            "std::basic_istream< char, std::char_traits< char > >",
            "std::istream")
        s = s.replace(
            "std::basic_ofstream< char, std::char_traits< char > >",
            "std::ostream")
        s = s.replace(
            "std::basic_ifstream< char, std::char_traits< char > >",
            "std::istream")

        # rename some types not renamed by gccxml (why ?)
        s = s.replace("itk::SerieUIDContainer", "std::vector< std::string >")
        s = s.replace("itk::FilenamesContainer", "std::vector< std::string >")

        if s.startswith("itk::") and not self.notWrappedRegExp.match(s):
            self.warn(
                4,
                "ITK type not wrapped, or currently not known: %s" %
                s,
                w)

        self.usedTypes.add(s)
        return s + end

    def load_idx(self, file_name):
        with open(file_name, "r") as f:
            for line in f:
                (full_name, alias, module) = \
                    re.findall(r'{(.*)} {(.*)} {(.*)}', line)[0]
                # workaround lack of :: prefix in idx files
                # TODO: would it be better to remove the :: prefix in the output of
                # pygccxml ?
                # full_name = "::"+full_name
                # normalize some basic type names
                full_name = self.normalize(full_name)

                if full_name in self.aliases:
                    # If the full_name key alreay exists, do not overwrite the
                    # value. load_idx() is called once before load_mdx(), making
                    # sure the first aliases loaded are the ones belonging to
                    # the current submodule (and the next load_idx() calls
                    # should not overwrite these aliases.
                    continue

                self.aliases[full_name] = alias
                # store the source of the def
                if alias in self.typedefSource and file_name != self.typedefSource[alias]:
                    self.warn(
                        7, "%s in %s is already defined in %s." %
                        (alias, file_name, self.typedefSource[alias]))
                else:
                    self.typedefSource[alias] = file_name

    def load_mdx(self, file_name):
        if file_name in self.mdx_loaded:
            # already loaded - no need to do it again
            return
        self.mdx_loaded.add(file_name)
        with open(file_name, "r") as f:
            lines = f.readlines()
        for line in lines:
            line_stripped = line.strip()
            if line.startswith('%') or line.isspace():
                # exclude the lines which are starting with % - that's not the idx
                # files
                pass
            elif line_stripped.endswith(".mdx"):
                self.load_mdx(os.path.dirname(file_name) + os.sep + line_stripped)
            elif line_stripped[:-4] == self.moduleName:
                continue
            else:
                self.load_idx(os.path.dirname(file_name) + os.sep + line_stripped)

    @staticmethod
    def normalize(name):
        name = name.replace("short unsigned int", "unsigned short")
        name = name.replace("long unsigned int", "unsigned long")
        name = name.replace("long long unsigned int", "unsigned long long")
        name = name.replace("short int", "short")
        name = name.replace("long int", "long")
        name = name.replace("long long int", "long long")
    #  name = name.replace("unsigned int", "unsigned")
        # normalize spaces
        name = " ".join(name.replace(',', ', ').split())
        return name

    def generate_class(self, typedef, indent=0):
        self.info("Generating interface for %s." % typedef.name)

        decls = pygccxml.declarations

        if not typedef.name.startswith("stdcomplex"):
            super_classes = []
            for super_class in getType(typedef).bases:
                super_classes.append(
                    "%s %s" %
                    (super_class.access,
                     self.get_alias(
                         super_class.related_class.decl_string)))
            s = ""
            if super_classes:
                s = " : " + ", ".join(super_classes)
            self.outputFile.write("  " * indent)
            self.outputFile.write("class %s%s {\n" % (typedef.name, s))

            # iterate over access
            for access in decls.ACCESS_TYPES.ALL:

                # the access type
                self.outputFile.write("  " * indent)
                self.outputFile.write("  %s:\n" % access)

                # warnings or no warning?
                w = access not in self.options.access_warnings

                # iterate over the members
                for member in getType(typedef).get_members(access=access):
                    if isinstance(member, decls.typedef.typedef_t):
                        self.warn(
                            51,
                            "Member typedef are not supported: %s" %
                            member.name,
                            w)
                    elif isinstance(member, decls.member_function_t):
                        self.generate_method(typedef, member, indent, w)
                    elif isinstance(member, decls.constructor_t):
                        self.generate_constructor(typedef, member, indent, w)
                    elif isinstance(member, decls.member_operator_t):
                        self.generate_method(typedef, member, indent, w)
                    elif isinstance(member, decls.destructor_t):
                        self.generate_destructor(typedef, member, indent, w)
                    elif isinstance(member, decls.enumeration_t):
                        self.generate_nested_enum(typedef, member, indent, w)
                    elif isinstance(member, decls.variable_t):
                        self.warn(
                            52,
                            "Member variables are not supported: %s" %
                            member.name,
                            w)
                    elif isinstance(member, decls.class_declaration.class_t):
                        self.warn(
                            53,
                            "Member classes are not supported: %s" %
                            member.name,
                            w)
                    elif isinstance(
                            member, decls.class_declaration.class_declaration_t):
                        self.warn(
                            53,
                            "Member classes are not supported: %s" %
                            member.name,
                            w)
                    elif isinstance(member, decls.casting_operator_t):
                        self.warn(
                            54,
                            "Member casting operators are not supported: %s" %
                            member.name,
                            w)
                    else:
                        self.warn(
                            50,
                            "Unknown member type: %s" %
                            repr(member),
                            w)

            # finally, close the class
            self.outputFile.write("  " * indent)
            self.outputFile.write("};\n\n\n")

        elif typedef.name == "stdcomplexD":
            self.outputFile.write(self.stdcomplex_headers["D"] + '\n')
        elif typedef.name == "stdcomplexF":
            self.outputFile.write(self.stdcomplex_headers["F"] + '\n')
        else:
            print('stdcomplex', typedef.name)
            # stdcomplex is too difficult to wrap in some cases. Only wrap the
            # constructor.
            self.outputFile.write("  " * indent)
            self.outputFile.write("class %s%s {\n" % (typedef.name, s))

            # iterate over access
            for access in pygccxml.declarations.ACCESS_TYPES.ALL:

                # the access type
                self.outputFile.write("  " * indent)
                self.outputFile.write("  %s:\n" % access)

                # warnings or no warning?
                w = access not in self.options.access_warnings
                for member in getType(typedef).get_members(access=access):
                    if isinstance(member, decls.constructor_t):
                        self.generate_constructor(typedef, member, indent, w)
                    elif isinstance(member, decls.destructor_t):
                        self.generate_destructor(typedef, member, indent, w)
            # finally, close the class
            self.outputFile.write("  " * indent)
            self.outputFile.write("};\n\n\n")

    def generate_constructor(self, typedef, constructor, indent, w):
        # iterate over the arguments
        args = []
        for arg in constructor.arguments:
            s = "%s %s" % (self.get_alias(self.getDeclarationString(arg), w), arg.name)
            # append the default value if it exists
            if arg.default_value:
                s += " = %s" % arg.default_value
            # and add the string to the arg list
            args.append(s)
        self.outputFile.write("  " * indent)
        self.outputFile.write("    %s(%s);\n" % (typedef.name, ", ".join(args)))

    def generate_destructor(self, typedef, destructor, indent, w):
        self.outputFile.write("  " * indent)
        self.outputFile.write("    ~%s();\n" % typedef.name)

    def generate_enum(self, typedef):
        name = typedef.name
        enum = getType(typedef)
        decl_string = typedef.decl_type.decl_string
        # extract the namespace to put it in c++ code. Without that, the code
        # generated by swig
        # is wrong because it doesn't include the namespace
        ns = "::".join(decl_string.split("::")[:-1])
        self.outputFile.write("%{\n")
        self.outputFile.write("using namespace %s;\n" % ns)
        self.outputFile.write("%}\n")
        content = [" %s = %i" % (key, value) for key, value in enum.values]
        self.outputFile.write("enum %s { %s };\n" % (name, ", ".join(content)))

    def generate_nested_enum(self, typedef, enum, indent, w):
        content = [" %s = %i" % (key, value) for key, value in enum.values]
        self.outputFile.write("  " * indent)
        self.outputFile.write("    enum %s { %s };\n" % (enum.name, ", ".join(content)))

    def generate_method(self, typedef, method, indent, w):
        self.info("Generating interface for method  '%s::%s'." %
            (typedef.name, method.name))
        # avoid the apply method for the class vnl_c_vector: the signature is
        # quite strange and currently confuse swig :-/
        if "(" in getType(method.return_type).decl_string:
            self.warn(
                1, "ignoring method not supported by swig '%s::%s'." %
                (typedef.name, method.name), w)
            return

        names = [
            "rBegin",
            "rEnd",
            "GetSpacingCallback",
            "GetOriginCallback",
            "Begin",
            "End"]

        if ((typedef.name.startswith('vnl_') and method.name in ["as_ref"])
                or (typedef.name.startswith('itk') and method.name in names)):
            self.warn(
                3, "ignoring black listed method '%s::%s'." %
                (typedef.name, method.name), w)
            return

        # iterate over the arguments
        args = []
        for arg in method.arguments:
            s = "%s %s" % (self.get_alias(self.getDeclarationString(arg), w), arg.name)
            if "(" in s:
                self.warn(
                    1, "ignoring method not supported by swig '%s::%s'." %
                    (typedef.name, method.name), w)
                return
            # append the default value if it exists
            if arg.default_value:
                s += " = %s" % arg.default_value
            # and add the string to the arg list
            args.append(s)

        # find the method decorators
        static = ""
        const = ""
        if method.has_static:
            static = "static "
        if method.has_const:
            const = " const"
        if method.virtuality != "not virtual":
            static += "virtual "
        if method.virtuality == "pure virtual":
            const += " = 0"

        self.outputFile.write("  " * indent)
        self.outputFile.write(
            "    %s%s %s(%s)%s;\n" %
            (static,
             self.get_alias(
                 self.getDeclarationString(
                     method.return_type),
                 w),
                method.name,
                ", ".join(args),
                const))

        # Check the method arguments for std::string passed by reference.
        # In this case, save the name of the argument in the applyFileNames list
        # for further usage.
        for arg in method.arguments:
            dtype = arg.decl_type
            if pygccxml.declarations.is_reference(dtype) and \
                pygccxml.declarations.is_const(
                    pygccxml.declarations.remove_reference(dtype)) is False and \
                    pygccxml.declarations.is_std_string(dtype):
                    self.applyFileNames.append(arg.name)


    def generate_headerfile(self, idxFile, wrappersNamespace):
        # and begin to write the output
        headerFile = StringIO()
        headerFile.write("// This file is automatically generated.\n")
        headerFile.write("// Do not modify this file manually.\n\n\n")

        langs = [
            "CHICKEN",
            "CSHARP",
            "GUILE",
            "JAVA",
            "LUA",
            "MODULA3",
            "MZSCHEME",
            "OCAML",
            "PERL",
            "PERL5",
            "PHP",
            "PHP4",
            "PHP5",
            "PIKE",
            "PYTHON",
            "R",
            "RUBY",
            "SEXP",
            "TCL",
            "XML"]

        # first, define the module
        # [1:-1] is there to drop the quotes
        for lang in langs:
            headerFile.write("#ifdef SWIG%s\n" % lang)
            headerFile.write("%%module %s%s\n" % (self.moduleName, lang.title()))
            headerFile.write("#endif\n")
        headerFile.write('\n')

        # add the includes
        # use a set to avoid putting many times the same include
        s = set()
        headerFile.write("%{\n")
        # the include files passed in option
        include = self.moduleName + 'SwigInterface.h'
        i = '#include "%s"' % include
        if i not in s:
            headerFile.write(i + '\n')
            s.add(i)
        headerFile.write("%}\n\n\n")

        # load the aliases files
        headerFile.write("%{\n")
        self.load_idx(idxFile)
        # and the idx files in the mdx ones
        for f in self.options.mdx:
            self.load_mdx(f)
        # iterate over all the typedefs in the _wrapping_::wrappers namespace
        # to fill the alias dict
        for typedef in wrappersNamespace.typedefs():  # allow_empty=True):
            s = getType(typedef).decl_string
            # drop the :: prefix - it make swig produce invalid code
            if s.startswith("::"):
                s = s[2:]
            if s not in self.aliases:
                self.warn(
                    2, "%s (%s) should be already defined in the idx files." %
                    (s, typedef.name))
                self.aliases[s] = typedef.name
                # declare the typedef
                headerFile.write("typedef %s %s;\n" % (s, typedef.name))

        headerFile.write("%}\n\n\n")

        return headerFile

    def generate_importfile(self, usedSources):
        # add the imports
        importFile = StringIO()
        for f in self.options.imports:
            importFile.write("%%import %s\n" % f)
        importFile.write("\n\n")

        for src in usedSources:
            importFile.write("%%import %s.i\n" % src)
        importFile.write('\n\n')
        return importFile

    def generate_includefile(self):
        # add the swig includes
        includeFile = StringIO()
        includeFile.write("%include itk.i\n")
        for f in options.swig_includes:
            includeFile.write("%%include %s\n" % f)
        includeFile.write("%%include %s\n" % (self.moduleName + "_ext.i"))
        includeFile.write('\n\n')
        return includeFile

    def generate_applyfile(self):
        # When a std::string is passed by reference, we need to add the %apply
        # line with the argument name, and the INOUT command.
        # Use a set() to remove duplicates, this will work event if we got
        # multiple functions with the same argument name in the same .i file
        # (swig should take care of it).
        applyFileNames = set(self.applyFileNames)
        # Apply file, for passing std::string as reference in methods
        applyFile = StringIO()
        for name in applyFileNames:
            applyFile.write(
                "%apply (std::string& INOUT) { std::string & " + name + "};\n")
        applyFile.write("\n\n")
        return applyFile

    def create_typedefheader(self, usedSources):
        # create the typedef header
        typedefFile = StringIO()
        typedefFile.write("#ifndef __%sSwigInterface_h\n" % self.moduleName)
        typedefFile.write("#define __%sSwigInterface_h\n" % self.moduleName)
        typedefInput = os.path.join(options.library_output_dir,
                                    self.moduleName + 'SwigInterface.h.in')
        with open(typedefInput, "r") as f:
            typedefFile.write(f.read() + '\n')
        for src in usedSources:
            typedefFile.write('#include "%sSwigInterface.h"\n' % src)
        typedefFile.write("#endif\n")
        typedefOutput = os.path.join(options.interface_output_dir,
                                     self.moduleName + 'SwigInterface.h')
        with open(typedefOutput, "w") as f:
            f.write(typedefFile.getvalue())

    def create_interfacefile(self, interfaceFile, idxFile, wrappersNamespace):
        headerFile = self.generate_headerfile(idxFile, wrappersNamespace)

        # iterate over all the typedefs in the _wrapping_::wrappers namespace
        # to build a list of classes with the dependecies
        # classes :: [(name, [dep_name], typedef)]
        classes = []
        for typedef in wrappersNamespace.typedefs():
            # begin a new class
            if isinstance(
                    getType(typedef),
                    pygccxml.declarations.class_declaration.class_t):

                classes.append((
                    typedef.name,
                    [self.get_alias(super_class.related_class.decl_string) for
                        super_class in getType(typedef).bases], typedef))
            elif isinstance(
                    getType(typedef),
                    pygccxml.declarations.enumeration.enumeration_t):
                # warn( 6, "Enum are currently supported only nested in a
                # class." )
                self.generate_enum(typedef)
            else:
                self.warn(
                    5, "Unknown type type: %s" % str(typedef.decl_type.declaration))

        # copy the classes in a new ordered list, according to the dependencies
        # classes is sorted to be sure to always get the same result everywhere
        name_local_classes = [c[0] for c in classes]
        classes = sorted(classes)
        name_already_in_typedefs = []
        typedefs = []
        while len(classes) != 0:
            nclasses = []
            for name, deps, typedef in classes:
                ok = True
                for d in deps:
                    if d in name_local_classes and d not in name_already_in_typedefs:
                        ok = False
                if ok:
                    name_already_in_typedefs.append(name)
                    typedefs.append(typedef)
                else:
                    nclasses.append((name, deps, typedef))
            classes = nclasses

        # now really generate the swig interface
        for typedef in typedefs:
            # begin a new class
            self.generate_class(typedef)

        if len(self.warnings) > 0 and self.options.warningError:
            sys.exit(1)

        # search the files to import
        usedSources = set()
        for alias in self.usedTypes:
            if alias in self.typedefSource:
                idxName = os.path.basename(self.typedefSource[alias])
                iName = idxName[:-len(".idx")]
                usedSources.add(iName)
        outputFileName = os.path.basename(interfaceFile)
        if outputFileName in usedSources:
            usedSources.remove(outputFileName)

        importFile = self.generate_importfile(usedSources)
        includeFile = self.generate_includefile()
        applyFile = self.generate_applyfile()

        self.create_typedefheader(usedSources)

        # finally, really write the output
        content = headerFile.getvalue() + importFile.getvalue() + \
            includeFile.getvalue() + applyFile.getvalue() + self.outputFile.getvalue()

        if self.options.keep and os.path.exists(interfaceFile):
            with open(interfaceFile, "r") as f:
                filecontent = f.read()

        if self.options.keep and os.path.exists(interfaceFile) and \
                filecontent == content:
            self.info("%s unchanged." % interfaceFile)
        else:
            self.info("Writing %s." % interfaceFile)
            with open(interfaceFile, "w") as f:
                f.write(content)


if __name__ == '__main__':
    argParser = ArgumentParser()
    argParser.add_argument(
        "--mdx",
        action="append",
        dest="mdx",
        default=[],
        metavar="FILE",
        help="master idx file to be used.")
    argParser.add_argument(
        "--import",
        action="append",
        dest="imports",
        default=[],
        metavar="FILE",
        help="File to be imported in the generated interface file.")
    argParser.add_argument(
        "--swig-include",
        action="append",
        dest="swig_includes",
        default=[],
        metavar="FILE",
        help=(
            "File to be included by swig (%include) in the generated "
            "interface file."))
    argParser.add_argument(
        "-w",
        "--disable-warning",
        action="append",
        dest="warnings",
        default=[],
        metavar="WARNING",
        help="Warning to be disabled.")
    argParser.add_argument(
        "-A",
        "--disable-access-warning",
        action="append",
        dest="access_warnings",
        default=[],
        metavar="LEVEL",
        help=(
            "Access level where warnings are disabled "
            "(public, protected, private)."))
    argParser.add_argument(
        "-W",
        "--warning-error",
        action="store_true",
        dest="warningError",
        help="Treat warnings as errors.")
    argParser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        dest="verbose",
        help="Log what is currently done.")
    argParser.add_argument(
        "-k",
        "--keep",
        action="store_true",
        dest="keep",
        help="Don't rewrite the output file if the content is unchanged.")
    argParser.add_argument(
        "-p",
        "--pygccxml-path",
        action="store",
        dest="pygccxml_path",
        help="Path to pygccxml")
    argParser.add_argument(
        "-g",
        "--castxml-path",
        action="store",
        dest="castxml_path",
        help="Path to castxml")
    argParser.add_argument(
        "-o",
        "--interface-output-dir",
        action="store",
        dest="interface_output_dir",
        help="Directory to write the Swig input files")
    argParser.add_argument(
        "-l",
        "--library-output-dir",
        action="store",
        dest="library_output_dir",
        help="Directory to read the xml abstract syntax tree input files")
    argParser.add_argument(
        "-s",
        "--submodule-order",
        action="store",
        dest="submodule_order",
        help="List of submodules that must be wrapped in the given order")
    options = argParser.parse_args()

    sys.path.insert(1, options.pygccxml_path)
    import pygccxml
    import logging
    # init the pygccxml stuff
    pygccxml.utils.loggers.cxx_parser.setLevel(logging.CRITICAL)
    pygccxml.declarations.scopedef_t.RECURSIVE_DEFAULT = False
    pygccxml.declarations.scopedef_t.ALLOW_EMPTY_MDECL_WRAPPER = True

    pygccxml_config = pygccxml.parser.config.xml_generator_configuration_t(
        xml_generator_path=options.castxml_path,
        xml_generator="castxml")

    moduleNames = []
    # The first mdx file is the master index file for this module.
    with open(options.mdx[0], 'r') as ff:
        for line in ff.readlines():
            stripped = line.strip()
            if line.startswith('%') or line.isspace():
                # exclude the lines which are starting with % - that's not the idx
                # files
                pass
            elif stripped.endswith(".mdx"):
                pass
            else:
                moduleName = stripped.rsplit('.')[0]
                if moduleName.startswith('(const char*)'):
                    moduleName = moduleName[len('(const char*)'):]
                moduleName = moduleName.strip('"')
                moduleNames.append(moduleName)

    def generate_wrapping_namespace(moduleName):
        xmlFilePath = os.path.join(options.library_output_dir,
                                   moduleName + '.xml')
        pygccxml_reader = pygccxml.parser.source_reader.source_reader_t(
            pygccxml_config)
        abstractSyntaxTree = pygccxml_reader.read_xml_file(xmlFilePath)
        globalNamespace = pygccxml.declarations.get_global_namespace(abstractSyntaxTree)
        wrappingNamespace = globalNamespace.namespace('_wrapping_')
        return wrappingNamespace.namespace('wrappers')

    wrappingNamespaces = dict()
    # Limit the number of cached, parsed abstract syntax trees to avoid very
    # high memory usage
    wrappingCacheLength = min(len(moduleNames), 20)
    for ii in range(wrappingCacheLength):
        moduleName = moduleNames[ii]
        wrappingNamespace = generate_wrapping_namespace(moduleName)
        wrappingNamespaces[moduleName] = wrappingNamespace

    for moduleName in moduleNames:
        if moduleName in wrappingNamespaces:
            wrappersNamespace = wrappingNamespaces[moduleName]
        else:
            wrappersNamespace = generate_wrapping_namespace(moduleName)

        idxFilePath = os.path.join(options.interface_output_dir,
                                   moduleName + '.idx')
        idx_generator = IdxGenerator(moduleName)
        idx_generator.create_idxfile(idxFilePath, wrappersNamespace)

    def generate_swig_input(moduleName):
        if moduleName in wrappingNamespaces:
            wrappersNamespace = wrappingNamespaces[moduleName]
        else:
            wrappersNamespace = generate_wrapping_namespace(moduleName)

        idxFilePath = os.path.join(options.interface_output_dir,
                                   moduleName + '.idx')
        swigInputFilePath = os.path.join(options.interface_output_dir,
                                   moduleName + '.i')

        swig_input_generator = SwigInputGenerator(moduleName, options)
        swig_input_generator.create_interfacefile(swigInputFilePath, idxFilePath,
                wrappersNamespace)

    if options.submodule_order:
        for moduleName in options.submodule_order.split(';'):
            generate_swig_input(moduleName)
            moduleNames.remove(moduleName)
    for moduleName in moduleNames:
        generate_swig_input(moduleName)



#!/usr/bin/env python

import argparse
import girder_client
from girder_client import GirderClient
import os
import fnmatch
import json
import mimetypes
from distutils.version import StrictVersion

if StrictVersion(girder_client.__version__) < StrictVersion("2.0.0"):
    raise Exception("Girder 2.0.0 or newer is required")

class GirderExternalDataCli(GirderClient):
    """
    A command line Python client for interacting with a Girder instance's
    RESTful api, specifically for performing uploads into a Girder instance.
    """
    def __init__(self, apiKey, objectStore):
        """initialization function to create a GirderCli instance, will attempt
        to authenticate with the designated Girder instance.
        """
        GirderClient.__init__(self,
                              apiUrl='https://data.kitware.com/api/v1')
        self.objectStore = objectStore
        self.authenticate(apiKey=apiKey)

    def content_link_upload(self, localFolder, parentId, ext='.sha512',
            parentType='folder', blacklist=['.git', '.ExternalData'],
            reuseExisting=True, dryRun=False):
        """Upload objects corresponding to CMake ExternalData content links.

        This will recursively walk down the tree and find content links ending
        with the specified extension and create a hierarchy on the server under
        the parentId.

        :param ext: Content link file extension.
        :param parentId: id of the parent in Girder or resource path.
        :param parentType: one of (collection,folder,user), default of folder.
        :param reuseExisting: bool whether to accept an existing item of
            the same name in the same location, or create a new one instead.
        :param dryRun: Do not actually upload any content.
        """
        parentId = self._checkResourcePath(parentId)
        localFolder = os.path.normpath(localFolder)
        for entry in os.listdir(localFolder):
            if entry in blacklist:
                print("Ignoring file %s as it is blacklisted" % entry)
                continue
            full_entry = os.path.join(localFolder, entry)
            if os.path.islink(full_entry):
                # os.walk skips symlinks by default
                print("Skipping file %s as it is a symlink" % entry)
                continue
            if os.path.isdir(full_entry):
                self._uploadFolderRecursive(
                    full_entry, parentId, parentType, ext,
                    reuseExisting=reuseExisting, blacklist=blacklist,
                    dryRun=dryRun)

    def _uploadContentLinkItem(self, name, content_link, folder,
            ext='.sha512', parentType='folder', dryRun=False,
            reuseExisting=False):
        """Upload objects corresponding to CMake ExternalData content links.

        This will upload the file with name, *name*, for the content link
        located at *content_link* to the Girder folder, *folder*.

        :param ext: Content link file extension.
        :param parentType: one of (collection,folder,user), default of folder.
        :param reuseExisting: bool whether to accept an existing item of
            the same name in the same location, or create a new one instead.
        :param dryRun: Do not actually upload any content.
        """
        content_link = os.path.normpath(content_link)
        if os.path.isfile(content_link) and \
                fnmatch.fnmatch(content_link, '*' + ext):
            if parentType != 'folder':
                raise Exception(('Attempting to upload an item under a %s.'
                                % parentType) +
                                ' Items can only be added to folders.')
            else:
                with open(content_link, 'r') as fp:
                    hash_value = fp.readline().strip()
                self._uploadAsItem(
                    name,
                    folder['_id'],
                    os.path.join(self.objectStore, hash_value),
                    reuseExisting=reuseExisting,
                    dryRun=dryRun)

    def _uploadFolderRecursive(self, localFolder, parentId, parentType,
                                 ext='.sha512',
                                 reuseExisting=False,
                                 blacklist=[],
                                 dryRun=False):
        """Function to recursively upload a folder and all of its descendants.
        :param localFolder: full path to local folder to be uploaded
        :param parentId: id of parent in Girder,
            where new folder will be added
        :param parentType: one of (collection, folder, user)
        :param leaf_folders_as_items: whether leaf folders should have all
        files uploaded as single items
        :param reuseExisting: boolean indicating whether to accept an existing
        item
        of the same name in the same location, or create a new one instead
        """
        localFolder = os.path.normpath(localFolder)
        filename = os.path.basename(localFolder)
        if filename in blacklist:
            print("Ignoring file %s as it is blacklisted" % filename)
            return

        # Do not add the folder if it does not contain any content links
        has_content_link = False
        for root, dirnames, filenames in os.walk(localFolder):
                for filename in fnmatch.filter(filenames, '*' + ext):
                        has_content_link = True
                        break
        if not has_content_link:
            return

        print('Creating Folder from %s' % localFolder)
        if dryRun:
            # create a dryRun placeholder
            folder = {'_id': 'dryRun'}
        elif localFolder == '.':
            folder = {'_id': parentId}
        else:
            folder = self.loadOrCreateFolder(
                os.path.basename(localFolder), parentId, parentType)

        for entry in sorted(os.listdir(localFolder)):
            if entry in blacklist:
                print("Ignoring file %s as it is blacklisted" % entry)
                continue
            full_entry = os.path.join(localFolder, entry)
            if os.path.islink(full_entry):
                # os.walk skips symlinks by default
                print("Skipping file %s as it is a symlink" % entry)
                continue
            elif os.path.isdir(full_entry):
                # At this point we should have an actual folder, so can
                # pass that as the parentType
                self._uploadFolderRecursive(
                    full_entry, folder['_id'], 'folder',
                    ext, reuseExisting=reuseExisting,
                    blacklist=blacklist, dryRun=dryRun)
            else:
                name = os.path.splitext(entry)[0]
                self._uploadContentLinkItem(name, full_entry, folder,
                        ext=ext, parentType=parentType, dryRun=dryRun,
                        reuseExisting=reuseExisting)

            if not dryRun:
                for callback in self._folderUploadCallbacks:
                    callback(folder, localFolder)


def main():
    parser = argparse.ArgumentParser(
        description='Upload CMake ExternalData content links to Girder')
    parser.add_argument(
        '--dry-run', action='store_true',
        help='will not write anything to Girder, only report on what would '
        'happen')
    parser.add_argument('--api-key', required=True, default=None)
    parser.add_argument('--local-folder', required=False,
                        default=os.path.join(os.path.dirname(__file__), '..',
                            '..'),
                        help='path to local target folder')
    # Default is ITK/ITKTestingData/Nightly
    parser.add_argument('--parent-id', required=False,
                        default='57b673388d777f10f269651c',
                        help='id of Girder parent target')
    parser.add_argument('--object-store', required=True,
                        help='Path to the CMake ExternalData object store')
    parser.add_argument(
        '--no-reuse', action='store_true',
        help='Don\'t reuse existing items of same name at same location')
    args = parser.parse_args()

    reuseExisting = not args.no_reuse
    gc = GirderExternalDataCli(args.api_key,
        objectStore=os.path.join(args.object_store, 'SHA512'))
    gc.content_link_upload(args.local_folder, args.parent_id,
            reuseExisting=reuseExisting, dryRun=args.dry_run)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Test BinaryDilateImageFilter
#

import sys
import itk
itk.auto_progress(2)

inputImage = sys.argv[1]
outputImage = sys.argv[2]
radiusValue = int(sys.argv[3])

PixelType = itk.UC
Dimension = 2

ImageType = itk.Image[PixelType, Dimension]

ReaderType = itk.ImageFileReader[ImageType]
reader = ReaderType.New()
reader.SetFileName(inputImage)

StructuringElementType = itk.FlatStructuringElement[Dimension]
structuringElement = StructuringElementType.Ball(radiusValue)

DilateFilterType = itk.BinaryDilateImageFilter[
    ImageType, ImageType, StructuringElementType]
dilateFilter = DilateFilterType.New()
dilateFilter.SetInput(reader.GetOutput())
dilateFilter.SetKernel(structuringElement)
dilateFilter.SetDilateValue(200)

WriterType = itk.ImageFileWriter[ImageType]
writer = WriterType.New()
writer.SetFileName(outputImage)
writer.SetInput(dilateFilter.GetOutput())

writer.Update()



#!/usr/bin/env python

#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from __future__ import print_function

usage = """usage: BuildHeaderTest.py <module_name> <module_source_path> <module_binary_path> <maximum_number_of_headers>

This script generates a a source file designed to check the headers in each
module.  The generated HeaderTest can be found in the module binary 'test'
directory in a file itk<module_name>HeaderTest#.cxx.  This contains a null
main(), but includes all the classes in the module.  The primary purpose of this
test is to make sure there are not missing module dependencies.  It also tests
for syntax and missing #include's.
"""

# Headers to not test because of dependecy issues, etc.
BANNED_HEADERS = set(('itkDynamicLoader.h', # This cannot be included when ITK_DYNAMIC_LOADING is OFF
    'itkExceptionObject.h', # There is a pre-processor check so people use itkMacro.h instead.
    'itkFFTWForwardFFTImageFilter.h',
    'itkFFTWInverseFFTImageFilter.h',
    'itkFFTWRealToHalfHermitianForwardFFTImageFilter.h',
    'itkFFTWHalfHermitianToRealInverseFFTImageFilter.h',
    'itkFFTWComplexToComplexFFTImageFilter.h',
    'itkFFTWCommon.h',
    'itkPyBuffer.h', # needs Python.h, etc
    'itkPyVnl.h', # needs Python.h, etc
    'itkVanHerkGilWermanErodeDilateImageFilter.h', # circular include's
    'itkBSplineDeformableTransform.h',   # deprecated
    'vtkCaptureScreen.h',  # these includes require VTK
    'itkBSplineDeformableTransformInitializer.h'))

HEADER = """/*=========================================================================
 *
 *  Copyright Insight Software Consortium
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *=========================================================================*/

// This file has been generated by BuildHeaderTest.py
// To regenerate, build the ITKHeaderTests target.
// This is a test to include each header file for Insight.

"""

TRAILER = """
#include <cstdlib> // needed for EXIT_SUCCESS macro

int main ( int , char* [] )
{

  return EXIT_SUCCESS;
}
"""

import glob
import os
import sys

if len(sys.argv) < 6:
    print(usage)
    sys.exit(1)

def main():
    module_name        = sys.argv[1]
    module_source_path = sys.argv[2]
    module_binary_path = sys.argv[3]
    maximum_number_of_headers = int(sys.argv[4])
    test_num           = int(sys.argv[5])

    # Get all the header files.
    include_dir = os.path.join(module_source_path, 'include')
    h_files = glob.glob(os.path.join(include_dir, '*.h'))
    h_files = [os.path.basename(h) for h in h_files]

    added_header_idx = maximum_number_of_headers * (test_num - 1)
    test_source_path = os.path.join(module_binary_path, 'test')
    if not os.path.exists(test_source_path):
        os.makedirs(test_source_path)
    test_source_file = os.path.join(test_source_path,
        str(module_name) + 'HeaderTest' + str(test_num) + '.cxx')

    test_src = open(test_source_file, 'w')
    try:
        test_src.write(HEADER)

        if added_header_idx + maximum_number_of_headers > len(h_files):
            max_idx = added_header_idx + len(h_files) % maximum_number_of_headers
        else:
            max_idx = added_header_idx + maximum_number_of_headers
        for i in range(added_header_idx, max_idx):
            # Use the .hxx if possible.
            hxx_file = h_files[i][:-1] + 'hxx'
            # Files that include VTK headers need to link to VTK.
            if h_files[i] in BANNED_HEADERS or h_files[i].lower().find('vtk') != -1:
                to_include = '// #include "' + h_files[i] + '" // Banned in BuildHeaderTest.py\n'
            elif os.path.exists(os.path.join(module_source_path, 'include',
                hxx_file)):
                to_include = '#include "' + hxx_file + '"\n'
            else:
                to_include = '#include "' + h_files[i] + '"\n'

            test_src.write(to_include)

        test_src.write(TRAILER)
    finally:
        test_src.close()

    return 0

if __name__ == "__main__":
    ret = main()
    sys.exit(ret)



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/


#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput1.png}
#     0

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput2.png}
#     1

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput3.png}
#     2

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput4.png}
#     3

from __future__ import print_function

import itk
from sys import argv, stderr, exit
itk.auto_progress(2)


if(len(argv) < 3):
    print((
        "Missing Parameters \n Usage: ResampleImageFilter.py inputImageFile "
        "outputImageFile [exampleAction={0,1,2,3}]"), file=stderr)
    exit(1)

exampleAction = 0

if(len(argv) >= 4):
    exampleAction = int(argv[3])

Dimension = 2
InputPixelType = itk.UC
OutputPixelType = itk.UC
InputImageType = itk.Image[InputPixelType, Dimension]
OutputImageType = itk.Image[OutputPixelType, Dimension]

ReaderType = itk.ImageFileReader[InputImageType]
CastType = itk.CastImageFilter[InputImageType, OutputImageType]
WriterType = itk.ImageFileWriter[OutputImageType]

reader = ReaderType.New()
cast = CastType.New()
writer = WriterType.New()

reader.SetFileName(argv[1])
writer.SetFileName(argv[2])

FilterType = itk.ResampleImageFilter[InputImageType, InputImageType]
filter = FilterType.New()

TransformType = itk.AffineTransform[itk.D, Dimension]

transform = TransformType.New()
filter.SetTransform(transform)

InterpolatorType = itk.NearestNeighborInterpolateImageFunction[
    InputImageType, itk.D]

interpolator = InterpolatorType.New()
filter.SetInterpolator(interpolator)

filter.SetDefaultPixelValue(0)

# spacing = itk.Vector[itk.D, Dimension]()
# spacing.SetElement(0, 1.0)
# spacing.SetElement(1, 1.0)
# filter.SetOutputSpacing( spacing )
filter.SetOutputSpacing(1.0)

# origin = itk.Point[itk.D, Dimension]()
# origin.SetElement(0, 0.0)
# origin.SetElement(1, 0.0)
# filter.SetOutputOrigin( origin )
filter.SetOutputOrigin([0.0, 0.0])

# size = itk.Size[Dimension]()
# size.SetElement(0, 300)
# size.SetElement(1, 300)
# filter.SetSize( size )
filter.SetSize(300)

filter.SetInput(reader.GetOutput())
cast.SetInput(filter.GetOutput())
writer.SetInput(cast.GetOutput())
writer.Update()

# translation = itk.Vector[itk.D, Dimension]()
# translation.SetElement(0, -30)
# translation.SetElement(1, -50)
# transform.Translate( translation, 0 )
transform.Translate([-30, -50], False)

if(exampleAction == 1):
    writer.Update()


filter.SetDefaultPixelValue(100)

if(exampleAction == 2):
    writer.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

import itk
from sys import argv


#
#  Check input parameters
#  INPUTS(fixedImage):  {BrainProtonDensitySliceBorder20.png}
#  INPUTS(movingImage): {BrainProtonDensitySliceShifted13x17y.png}
#
if len(argv) < 4:
    print 'Missing Parameters'
    print 'Usage: ImageRegistration4.py fixedImageFile  movingImageFile outputImagefile'
    exit()


#
#  Define data types
#
FixedImageType  = itk.Image[itk.F, 2]
MovingImageType = itk.Image[itk.F, 2]
OutputImageType = itk.Image[itk.UC,2]
TransformType   = itk.TranslationTransform[itk.D, 2]


#
#  Read the fixed and moving images using filenames
#  from the command line arguments
#
fixedImageReader  = itk.ImageFileReader[FixedImageType].New()
movingImageReader = itk.ImageFileReader[MovingImageType].New()

fixedImageReader.SetFileName(  argv[1] )
movingImageReader.SetFileName( argv[2] )

fixedImageReader.Update()
movingImageReader.Update()

fixedImage  = fixedImageReader.GetOutput()
movingImage = movingImageReader.GetOutput()


#
#  Instantiate the classes for the registration framework
#
registration = itk.ImageRegistrationMethod[FixedImageType, MovingImageType].New()
imageMetric  = itk.MattesMutualInformationImageToImageMetric[FixedImageType, MovingImageType].New()
transform    = TransformType.New()
optimizer    = itk.RegularStepGradientDescentOptimizer.New()
interpolator = itk.LinearInterpolateImageFunction[FixedImageType, itk.D].New()

imageMetric.SetNumberOfHistogramBins( 20 );
imageMetric.SetNumberOfSpatialSamples( 10000 );

registration.SetOptimizer(    optimizer    )
registration.SetTransform(    transform    )
registration.SetInterpolator( interpolator )
registration.SetMetric(       imageMetric  )

registration.SetFixedImage(  fixedImage  )
registration.SetMovingImage( movingImage )

registration.SetFixedImageRegion( fixedImage.GetBufferedRegion() )

transform.SetIdentity()
initialParameters = transform.GetParameters()

registration.SetInitialTransformParameters( initialParameters )


#
# Iteration Observer
#
def iterationUpdate():
    currentParameter = transform.GetParameters()
    print "M: %f   P: %f %f " % ( optimizer.GetValue(),
                        currentParameter.GetElement(0),
                        currentParameter.GetElement(1) )

iterationCommand = itk.PyCommand.New()
iterationCommand.SetCommandCallable( iterationUpdate )
optimizer.AddObserver( itk.IterationEvent(), iterationCommand )


#
#  Define optimizer parameters
#
optimizer.SetMaximumStepLength(  4.00 )
optimizer.SetMinimumStepLength(  0.01 )
optimizer.SetNumberOfIterations( 200  )

print "Starting registration"


#
#  Start the registration process
#
registration.Update()


#
# Get the final parameters of the transformation
#
finalParameters = registration.GetLastTransformParameters()

print "Final Registration Parameters "
print "Translation X =  %f" % (finalParameters.GetElement(0),)
print "Translation Y =  %f" % (finalParameters.GetElement(1),)


#
# Now, we use the final transform for resampling the
# moving image.
#
resampler = itk.ResampleImageFilter[MovingImageType, FixedImageType].New()
resampler.SetTransform( transform    )
resampler.SetInput(     movingImage  )

region = fixedImage.GetLargestPossibleRegion()

resampler.SetSize( region.GetSize() )

resampler.SetOutputSpacing( fixedImage.GetSpacing() )
resampler.SetOutputDirection( fixedImage.GetDirection() )
resampler.SetOutputOrigin(  fixedImage.GetOrigin()  )
resampler.SetDefaultPixelValue( 100 )

outputCast = itk.RescaleIntensityImageFilter[FixedImageType, OutputImageType].New()
outputCast.SetInput(resampler.GetOutput())


#
#  Write the resampled image
#
writer = itk.ImageFileWriter[OutputImageType].New()

writer.SetFileName( argv[3] )
writer.SetInput( outputCast.GetOutput() )
writer.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/


class itkCType:
    __cTypes__ = {}

    def __init__(self, name, shortName):
        self.name = name
        self.short_name = shortName

        itkCType.__cTypes__[self.name] = self

    def __del__(self):
        try:
            del itkCType.__cTypes__[self.name]
        except:
            pass

    def __repr__(self):
        return "<itkCType %s>" % self.name

    def GetCType(name):
        aliases = {'short': 'signed short',
            'int': 'signed int',
            'long': 'signed long',
            'long long': 'signed long long'}
        if name in aliases:
            name = aliases[name]
        try:
            return(itkCType.__cTypes__[name])
        except KeyError:
            return(None)
    GetCType = staticmethod(GetCType)


F = itkCType("float", "F")
D = itkCType("double", "D")
LD = itkCType("long double", "LD")
UC = itkCType("unsigned char", "UC")
US = itkCType("unsigned short", "US")
UI = itkCType("unsigned int", "UI")
UL = itkCType("unsigned long", "UL")
ULL = itkCType("unsigned long long", "ULL")
SC = itkCType("signed char", "SC")
SS = itkCType("signed short", "SS")
SI = itkCType("signed int", "SI")
SL = itkCType("signed long", "SL")
SLL = itkCType("signed long long", "SLL")
B = itkCType("bool", "B")



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the MedianImageFilter
#

import itk
from sys import argv

reader = itk.ImageFileReader.IUC2.New(FileName=argv[1])

# test the deduction of the template parameter from the input
filter = itk.MedianImageFilter.New(reader)
watcher = itk.XMLFilterWatcher(filter, "filter")

# test the update of the filter with the __call__ operator
# and the setting of parameter inside it
filter(Radius=eval(argv[3]))

# test the write method
itk.write(filter, argv[2])



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of DicomImageIO for reading a single DICOM slice, rescale
#  the intensities and save it in a different file format.
#

from InsightToolkit import *

from sys import argv

#
# Reads an image in  16bits/pixel
# and save it as      8bits/pixel
#
reader = itkImageFileReaderUS2_New()
writer = itkImageFileWriterUC2_New()

dicomIO = itkDicomImageIO_New()

reader.SetImageIO(dicomIO.GetPointer())

filter = itkRescaleIntensityImageFilterUS2UC2_New()

filter.SetOutputMinimum(0)
filter.SetOutputMaximum(255)

filter.SetInput(reader.GetOutput())
writer.SetInput(filter.GetOutput())

reader.SetFileName(argv[1])
writer.SetFileName(argv[2])

writer.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the CannyEdgeDetectionImageFilter
#

from __future__ import print_function

import itk
from sys import argv, stderr, exit
itk.auto_progress(2)

if len(argv) < 3:
    print((
        "Usage: CannyEdgeDetectionImageFilter.py inputImage outputImage "
        "[variance]"), file=stderr)
    exit(1)

variance = 2.0
if len(argv) > 3:
    variance = float(argv[3])
    print(variance)

reader = itk.ImageFileReader.IF2.New(FileName=argv[1])
filter = itk.CannyEdgeDetectionImageFilter.IF2IF2.New(
    reader,
    Variance=variance)
outputCast = itk.RescaleIntensityImageFilter.IF2IUC2.New(
    filter,
    OutputMinimum=0,
    OutputMaximum=255)
itk.write(outputCast, argv[2])



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from InsightToolkit import *
from numarray import *
from sys import argv

reader = itkImageFileReaderUC2_New()

connector = itkPyBufferUC2_New()

reader.SetFileName( argv[1] )

reader.Update()

print "ready to convert image into array"

buffer = connector.GetArrayFromImage( reader.GetOutput() )

writer = itkImageFileWriterUC2_New()

writer.SetFileName( argv[2] )

print "ready to convert array into image"

writer.SetInput( connector.GetImageFromArray( buffer ) )

writer.Update()





#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from __future__ import print_function

import time

start = time.time()
import itk
itk.force_load()
print("load:", time.time() - start)

img = itk.Image.UC2.New(Regions=(10, 20))
img.Allocate()


def index(it):
    idx = itk.Index[2]()
    for dummy in range(0, it):
        img.GetPixel(idx)


def index2(it):
    idx = itk.Index[2]()
    for dummy in range(0, it):
        idx[0] = 0
        idx[1] = 0
        img.GetPixel(idx)


def integer(it):
    for dummy in range(0, it):
        img.GetPixel(0)


def pylist(it):
    l = [0, 0]
    for dummy in range(0, it):
        img.GetPixel(l)


def pytuple(it):
    l = (0, 0)
    for dummy in range(0, it):
        img.GetPixel(l)


def new(it):
    for dummy in range(0, it):
        lo = itk.Image.UC2.__New_orig__()


def extended_new(it):
    for dummy in range(0, it):
        lo = itk.Image.UC2.New()


def run(f, it):
    start = time.time()
    f(it)
    return time.time() - start


it = 1000000
print("index:", run(index, it))
print("index2:", run(index2, it))
print("integer:", run(integer, it))
print("list:", run(pylist, it))
print("tuple:", run(pytuple, it))
print("new:", run(new, 100000))
print("extended_new:", run(extended_new, 100000))



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the VoronoiSegmentationImageFilter.
#

from InsightToolkit import *

from sys import argv


readerInput = itkImageFileReaderUC2_New()
readerPrior = itkImageFileReaderUC2_New()

readerInput.SetFileName( argv[1] )
readerPrior.SetFileName( argv[2] )

readerInput.Update()
readerPrior.Update()

filter  = itkVoronoiSegmentationImageFilterUC2UC2UC2_New()

filter.SetInput(   readerInput.GetOutput() )
filter.TakeAPrior( readerPrior.GetOutput() )

filter.SetMeanPercentError( eval( argv[4] )  )
filter.SetSTDPercentError( eval( argv[5] )  )

writer = itkImageFileWriterUC2_New()
writer.SetFileName( argv[3] )
writer.SetInput( filter.GetOutput() )

writer.Update()




#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

import itk

Dimension = 2

index = itk.Index[Dimension]()
index[0] = 1
index[1] = 2

offset = itk.Offset[Dimension]()
offset.Fill(1)

a = index + offset
b = offset + index
c = index + index
d = offset + offset
e = index - offset

assert a[0] == 2
assert a[1] == 3

assert b[0] == 2
assert b[1] == 3

assert c[0] == 2
assert c[1] == 4

assert d[0] == 2
assert d[1] == 2

assert e[0] == 0
assert e[1] == 1



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput1.png}
#     0

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput2.png}
#     1

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput3.png}
#     2

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ResampleImageFilterOutput4.png}
#     3

from __future__ import print_function

import itk
from sys import argv, stderr, exit

itk.auto_progress(2)

# if( len(argv) < 3 ):
#   print("""Missing Parameters
# Usage: ResampleImageFilter.py inputImageFile outputImageFile
# [exampleAction={0,1,2,3}]""", file=stderr)
#   exit(1)

dim = 2
SOType = itk.SpatialObject[dim]
InternalImageType = itk.Image[itk.F, dim]
OutputPixelType = itk.UC
OutputImageType = itk.Image[OutputPixelType, dim]

ellipse = itk.EllipseSpatialObject[dim].New(Radius=[10, 5])
ellipse.GetObjectToParentTransform().SetOffset([20, 20])
ellipse.ComputeObjectToWorldTransform()

box = itk.BoxSpatialObject[dim].New(Size=20)
box.GetObjectToParentTransform().SetOffset([20, 40])
box.ComputeObjectToWorldTransform()

gaussian = itk.GaussianSpatialObject[dim].New(Radius=100)
gaussian.GetObjectToParentTransform().SetOffset([60, 60])
gaussian.GetObjectToParentTransform().SetScale(10)
gaussian.ComputeObjectToWorldTransform()

group = itk.GroupSpatialObject[dim].New()
group.AddSpatialObject(ellipse)
group.AddSpatialObject(box)
group.AddSpatialObject(gaussian)

filter = itk.SpatialObjectToImageFilter[SOType, InternalImageType].New(
    group, Size=[100, 100], UseObjectValue=True)
filter.Update()  # required ?!

rescale = itk.RescaleIntensityImageFilter[
    InternalImageType,
    OutputImageType].New(
    filter,
    OutputMinimum=itk.NumericTraits[OutputPixelType].NonpositiveMin(),
    OutputMaximum=itk.NumericTraits[OutputPixelType].max())

itk.write(rescale, argv[1])



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

import InsightToolkit as itk
import sys

reader = itk.itkImageFileReaderF2_New()
reader.SetFileName( sys.argv[1] )

diffusion = itk.itkGradientAnisotropicDiffusionImageFilterF2F2_New()
diffusion.SetInput(reader.GetOutput())
diffusion.SetTimeStep(0.0625)
diffusion.SetConductanceParameter(9.0)
diffusion.SetNumberOfIterations(  5 );

gradient = itk.itkGradientMagnitudeImageFilterF2F2_New()
gradient.SetInput(diffusion.GetOutput())

watershed = itk.itkWatershedImageFilterF2_New()
watershed.SetInput(gradient.GetOutput())
watershed.SetThreshold(0.01)
watershed.SetLevel(0.2)

writer = itk.itkImageFileWriterUL2_New()
writer.SetFileName( sys.argv[2] )
writer.SetInput( watershed.GetOutput()  )
writer.Update()






#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from __future__ import print_function

import inspect
import os
import re
import sys
import types
import collections
if sys.version_info >= (3, 4):
    import importlib
else:
    import imp
import warnings
import itkConfig
import itkBase
import itkLazy
from itkTypes import itkCType


def registerNoTpl(name, cl):
    """Register a class without template

    It can seem not useful to register classes without template (and it wasn't
    useful until the SmartPointer template was generated), but those classes
    can be used as template argument of classes with template.
    """
    itkTemplate.__templates__[normalizeName(name)] = cl


def normalizeName(name):
    """Normalize the class name to remove ambiguity

    This function removes the white spaces in the name, and also
    remove the pointer declaration "*" (it have no sense in python) """

    name = name.replace(" ", "")
    name = name.replace("*", "")

    return name


class itkTemplate(object):

    """This class manages access to available template arguments of a C++ class.

    This class is generic and does not give help on the methods available in
    the instantiated class. To get help on a specific ITK class, instantiate an
    object of that class.

    e.g.: median = itk.MedianImageFilter[ImageType, ImageType].New()
          help(median)

    There are two ways to access types:

    1. With a dict interface. The user can manipulate template parameters
    similarly to C++, with the exception that the available parameters sets are
    chosen at compile time. It is also possible, with the dict interface, to
    explore the available parameters sets.
    2. With object attributes. The user can easily find the available parameters
    sets by pressing tab in interperter like ipython
    """
    __templates__ = collections.OrderedDict()
    __class_to_template__ = {}
    __named_templates__ = {}
    __doxygen_root__ = itkConfig.doxygen_root

    def __new__(cls, name):
        # Singleton pattern: we only make a single instance of any Template of
        # a given name. If we have already made the instance, just return it
        # as-is.
        if name not in cls.__named_templates__:
                new_instance = object.__new__(cls)
                new_instance.__name__ = name
                new_instance.__template__ = collections.OrderedDict()
                cls.__named_templates__[name] = new_instance
        return cls.__named_templates__[name]

    def __add__(self, paramSetString, cl):
        """Add a new argument set and the resulting class to the template.

        paramSetString is the C++ string which defines the parameters set.
        cl is the class which corresponds to the couple template-argument set.
        """
        # recreate the full name and normalize it to avoid ambiguity
        normFullName = normalizeName(
            self.__name__ + "<" + paramSetString + ">")

        # the full class should not be already registered. If it is, there is a
        # problem somewhere so warn the user so he can fix the problem
        if normFullName in itkTemplate.__templates__:
            message = (
                "Template %s\n already defined as %s\n is redefined "
                "as %s") % (normFullName, self.__templates__[normFullName], cl)
            warnings.warn(message)
        # register the class
        itkTemplate.__templates__[normFullName] = cl

        # __find_param__ will parse the paramSetString and produce a list of
        # the same parameters transformed in corresponding python classes.
        # we transform this list in tuple to make it usable as key of the dict
        param = tuple(self.__find_param__(paramSetString))

        # once again, warn the user if the tuple of parameter is already
        # defined so he can fix the problem
        if param in self.__template__:
            message = "Warning: template already defined '%s'" % normFullName
            warnings.warn(message)
        # and register the parameter tuple
        self.__template__[param] = cl

        # add in __class_to_template__ dictionary
        itkTemplate.__class_to_template__[cl] = (self, param)

        # now populate the template
        # 2 cases:
        # - the template is a SmartPointer. In that case, the attribute name
        # will be the full real name of the class without the itk prefix and
        # _Pointer suffix
        # - the template is not a SmartPointer. In that case, we keep only the
        # end of the real class name which is a short string discribing the
        # template arguments (for example IUC2)
        if cl.__name__.startswith("itk"):
            if cl.__name__.endswith("_Pointer"):
                # it's a SmartPointer
                attributeName = cl.__name__[len("itk"):-len("_Pointer")]
            else:
                # it's not a SmartPointer
                # we need to now the size of the name to keep only the suffix
                # short name does not contain :: and nested namespace
                # itk::Numerics::Sample -> itkSample
                shortNameSize = len(re.sub(r':.*:', '', self.__name__))
                attributeName = cl.__name__[shortNameSize:]
        elif cl.__name__.startswith("vcl_complex"):
            raise AttributeError('former vcl_complex, handling ' + str(cl.__name__))
            # C++ name is likely to be std::complex here, instead of the
            # expected vcl_complex
            attributeName = cl.__name__[len("vcl_complex"):]
        else:
            shortName = re.sub(r':.*:', '', self.__name__)

            if not cl.__name__.startswith(shortName):
                shortName = re.sub(r'.*::', '', self.__name__)

            attributeName = cl.__name__[len(shortName):]

        if attributeName[0].isdigit():
            # the attribute name can't start with a number
            # add a single x before it to build a valid name.
            # Adding an underscore would hide the attributeName in IPython
            attributeName = "x" + attributeName

        # add the attribute to this object
        self.__dict__[attributeName] = cl

    def __find_param__(self, paramSetString):
        """Find the parameters of the template.

        paramSetString is the C++ string which defines the parameters set.

        __find_param__ returns a list of itk classes, itkCType, and/or numbers
        which correspond to the parameters described in paramSetString.
        The parameters MUST have been registered before calling this method,
        or __find_param__ will return a string and not the wanted object, and
        will display a warning. Registration order is important.

        This method is not static only to be able to display the template name
        in the warning.
        """
        # split the string in a list of parameters
        paramStrings = []
        inner = 0
        part = paramSetString.split(",")
        for elt in part:
            if inner == 0:
                paramStrings.append(elt)
            else:
                paramStrings[-1] += "," + elt
            inner += elt.count("<") - elt.count(">")

        # convert all string parameters into classes (if possible)
        parameters = []
        for param in paramStrings:
            # the parameter need to be normalized several time below
            # do it once here
            param = param.strip()
            paramNorm = normalizeName(param)

            if paramNorm in itkTemplate.__templates__:
                # the parameter is registered.
                # just get the really class form the dictionary
                param = itkTemplate.__templates__[paramNorm]

            elif itkCType.GetCType(param):
                # the parameter is a c type
                # just get the itkCtype instance
                param = itkCType.GetCType(param)

            elif paramNorm.isdigit():
                # the parameter is a number
                # convert the string to a number !
                param = int(param)

            elif paramNorm == "true":
                param = True
            elif paramNorm == "false":
                param = False

            else:
                # unable to convert the parameter
                # use it without changes, but display a warning message, to
                # incite developer to fix the problem
                message = (
                    "Warning: Unknown parameter '%s' in "
                    "template '%s'" % (param, self.__name__))
                warnings.warn(message)

            parameters.append(param)

        return parameters

    def __getitem__(self, parameters):
        """Return the class which corresponds to the given template parameters.

        parameters can be:
            - a single parameter (Ex: itk.Index[2])
            - a list of elements (Ex: itk.Image[itk.UC, 2])
        """

        parameters_type = type(parameters)
        if not parameters_type is tuple and not parameters_type is list:
            # parameters is a single element.
            # include it in a list to manage the 2 cases in the same way
            parameters = [parameters]

        cleanParameters = []
        for param in parameters:
            # In the case of itk class instance, get the class
            name = param.__class__.__name__
            isclass = inspect.isclass(param)
            if not isclass and name[:3] == 'itk' and name != "itkCType":
                param = param.__class__

            # append the parameter to the list. If it's not a supported type,
            # it is not in the dictionary and we will raise an exception below
            cleanParameters.append(param)

        try:
            return(self.__template__[tuple(cleanParameters)])
        except:
            self._LoadModules()
            try:
                return(self.__template__[tuple(cleanParameters)])
            except:
                raise KeyError(
                    'itkTemplate : No template %s for the %s class' %
                    (str(parameters), self.__name__))

    def __repr__(self):
        return '<itkTemplate %s>' % self.__name__

    def __getattr__(self, attr):
        """Support for lazy loading."""
        self._LoadModules()
        return object.__getattribute__(self, attr)


    def _LoadModules(self):
        """Loads all the module that may have not been loaded by the lazy loading system.

        If multiple modules use the same object, the lazy loading system is only going to
        load the module in which the object belongs. The other modules will be loaded only when necessary.
        """
        name=self.__name__.split('::')[-1] # Remove 'itk::' or 'itk::Function::'
        modules = itkBase.lazy_attributes[name]
        for module in modules:
            # find the module's name in sys.modules, or create a new module so named
            if sys.version_info >= (3, 4):
                this_module = sys.modules.setdefault(module, types.ModuleType(module))
            else:
                this_module = sys.modules.setdefault(module, imp.new_module(module))
            namespace = {}
            if not hasattr(this_module, '__templates_loaded'):
                itkBase.LoadModule(module, namespace)

    def __dir__(self):
        """Returns the list of the attributes available in the current template.

        This loads all the modules that might be required by this template first,
        and then returns the list of attributes. It is used when dir() is called
        or when it tries to autocomplete attribute names.
        """
        self._LoadModules()

        def get_attrs(obj):
            if not hasattr(obj, '__dict__'):
                return []  # slots only
            if sys.version_info >= (3, 0):
              dict_types = (dict, types.MappingProxyType)
            else:
              dict_types = (dict, types.DictProxyType)
            if not isinstance(obj.__dict__, dict_types):
                raise TypeError("%s.__dict__ is not a dictionary"
                                "" % obj.__name__)
            return obj.__dict__.keys()

        def dir2(obj):
            attrs = set()
            if not hasattr(obj, '__bases__'):
                # obj is an instance
                if not hasattr(obj, '__class__'):
                    # slots
                    return sorted(get_attrs(obj))
                klass = obj.__class__
                attrs.update(get_attrs(klass))
            else:
                # obj is a class
                klass = obj

            for cls in klass.__bases__:
                attrs.update(get_attrs(cls))
                attrs.update(dir2(cls))
            attrs.update(get_attrs(obj))
            return list(attrs)

        return dir2(self)


    def __call__(self, *args, **kwargs):
        filt = self.New(*args, **kwargs)
        try:
            filt.UpdateLargestPossibleRegion()
            img = filt.GetOutput()
        except AttributeError:
            img = filt
        return img

    def New(self, *args, **kwargs):
        """Instantiate the template with a type implied from its input.

        Template type specification can be avoided by assuming that the type's
        first template argument should have the same type as its primary input.
        This is generally true. If it is not true, then specify the types
        explicitly.

        For example, instead of the explicit type specification::

          median = itk.MedianImageFilter[ImageType, ImageType].New()
          median.SetInput(reader.GetOutput())

        call::

          median = itk.MedianImageFilter.New(Input=reader.GetOutput())

        or, the shortened::

          median = itk.MedianImageFilter.New(reader.GetOutput())

        or:

          median = itk.MedianImageFilter.New(reader)"""
        import itk
        keys = self.keys()
        cur = itk.auto_pipeline.current
        if self.__name__ == "itk::ImageFileReader":
            return self._NewImageFileReader(*args, **kwargs)
        primary_input_methods = ('Input', 'InputImage', 'Input1')
        if len(args) != 0:
            # try to find a type suitable for the primary input provided
            input_type = output(args[0]).__class__
            keys = [k for k in keys if k[0] == input_type]
        elif set(primary_input_methods).intersection(kwargs.keys()):
            for method in primary_input_methods:
                if method in kwargs:
                    input_type = output(kwargs[method]).__class__
                    keys = [k for k in keys if k[0] == input_type]
                    break
        elif cur is not None and len(cur) != 0:
            # try to find a type suitable for the input provided
            input_type = output(cur).__class__
            keys = [k for k in keys if k[0] == input_type]

        if len(keys) == 0:
            raise RuntimeError("No suitable template parameter can be found.")
        return self[list(keys)[0]].New(*args, **kwargs)

    def _NewImageFileReader(self, *args, **kwargs):
        primaryInputMethods = ('FileName',)
        inputFileName = ''
        if len(args) != 0:
            # try to find a type suitable for the primary input provided
            inputFileName = args[0]
        elif set(primaryInputMethods).intersection(kwargs.keys()):
            for method in primaryInputMethods:
                if method in kwargs:
                    inputFileName = kwargs[method]
                    break
        if not inputFileName:
            raise RuntimeError("No FileName specified.")
        import itk
        imageIO = itk.ImageIOFactory.CreateImageIO( inputFileName, itk.ImageIOFactory.ReadMode )
        if not imageIO:
            raise RuntimeError("No ImageIO is registered to handle the given file.")
        componentTypeDic= {"float": itk.F, "double": itk.D,
        "unsigned_char": itk.UC, "unsigned_short": itk.US, "unsigned_int": itk.UI,
        "unsigned_long": itk.UL, "unsigned_long_long": itk.ULL, "char": itk.SC, "short": itk.SS,
        "int": itk.SI, "long": itk.SL, "long_long": itk.SLL}
        # Read the metadata from the image file.
        imageIO.SetFileName( inputFileName )
        imageIO.ReadImageInformation()
        dimension = imageIO.GetNumberOfDimensions()
        componentAsString = imageIO.GetComponentTypeAsString(imageIO.GetComponentType())
        component = componentTypeDic[componentAsString]
        pixel = imageIO.GetPixelTypeAsString(imageIO.GetPixelType())
        PixelType = itkTemplate._pixelTypeFromIO(pixel, component, dimension)
        ImageType = itk.Image[PixelType, dimension]
        ReaderType = itk.ImageFileReader[ImageType]
        return ReaderType.New(*args, **kwargs)

    @staticmethod
    def _pixelTypeFromIO(pixel, component, dimension):
        import itk
        if pixel == 'scalar':
            PixelType = component
        elif pixel == 'rgb':
            PixelType = itk.RGBPixel[component]
        elif pixel == 'rgba':
            PixelType = itk.RGBAPixel[component]
        elif pixel == 'offset':
            PixelType = itk.Offset[dimension]
        elif pixel == 'vector':
            PixelType = itk.Vector[component, dimension]
        elif pixel == 'point':
            PixelType = itk.Point[component, dimension]
        elif pixel == 'covariant_vector':
            PixelType = itk.CovariantVector[component, dimension]
        elif pixel == 'symmetric_second_rank_tensor':
            PixelType = itk.SymmetricSecondRankTensor[component, dimension]
        elif pixel == 'diffusion_tensor_3D':
            PixelType = itk.DiffusionTensor3D[component]
        elif pixel == 'complex':
            PixelType = itk.complex[component]
        elif pixel == 'fixed_array':
            PixelType = itk.FixedArray[component, dimension]
        elif pixel == 'matrix':
            PixelType = itk.Matrix[component, dimension, dimension]
        else:
            raise RuntimeError("Unknown pixel type: %s." % pixel)
        return PixelType

    def keys(self):
        return self.__template__.keys()

    # everything after this comment is for dict interface
    # and is a copy/paste from DictMixin
    # only methods to edit dictionary are not there
    def __iter__(self):
        for k in self.keys():
            yield k

    def __contains__(self, key):
        return key in self

    # third level takes advantage of second level definitions
    def iteritems(self):
        for k in self:
            yield (k, self[k])

    def iterkeys(self):
        return self.__iter__()

    # fourth level uses definitions from lower levels
    def itervalues(self):
        for _, v in self.iteritems():
            yield v

    def values(self):
        return [v for _, v in self.iteritems()]

    def items(self):
        return list(self.iteritems())

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def __len__(self):
        return len(self.keys())

    def GetTypes(self):
        """Helper method which prints out the available template parameters."""

        print("<itkTemplate %s>" % self.__name__)
        print("Options:")
        for tp in self.GetTypesAsList():
            print("  " + str(tp).replace("(", "[").replace(")", "]"))

    def GetTypesAsList(self):
        """Helper method which returns the available template parameters."""

        # Make a list of allowed types, and sort them
        ctypes = []
        classes = []
        others = []

        for key_tuple in self.__template__:
            key = str(key_tuple)
            if "itkCType" in key:
                ctypes.append(key)
            elif "class" in key:
                classes.append(key)
            else:
                others.append(key)
        # Sort the lists
        ctypes = sorted(ctypes)
        classes = sorted(classes)
        others = sorted(others)

        return ctypes + classes + others


# create a new New function which accepts parameters
def New(self, *args, **kargs):
    import itk

    itk.set_inputs(self, args, kargs)

    # now, try to add observer to display progress
    if "auto_progress" in kargs.keys():
        if kargs["auto_progress"] in [True, 1]:
            callback = itk.terminal_progress_callback
        elif kargs["auto_progress"] == 2:
            callback = itk.simple_progress_callback
        else:
            callback = None
    elif itkConfig.ProgressCallback:
        callback = itkConfig.ProgressCallback
    else:
        callback = None

    if callback and not issubclass(self.__class__, itk.Command):
        try:
            name = self.__class__.__name__

            def progress():
                # self and callback are kept referenced with a closure
                callback(name, self.GetProgress())

            self.AddObserver(itk.ProgressEvent(), progress)
        except:
            # it seems that something goes wrong...
            # as this feature is designed for prototyping, it's not really a
            # problem if an object doesn't have progress reporter, so adding
            # reporter can silently fail
            pass

    if itkConfig.NotInPlace and "SetInPlace" in dir(self):
        self.SetInPlace(False)

    if itk.auto_pipeline.current is not None:
        itk.auto_pipeline.current.connect(self)

    return self


def output(input):
    try:
        img = input.GetOutput()
    except AttributeError:
        img = input
    return img


def image(input):
    warnings.warn("WrapITK warning: itk.image() is deprecated. "
            "Use itk.output() instead.")
    return output(input)



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

import itk

# To verify that the lazy loading system works, we try to load
# a float image (declared in the module ITKCommon)
image=itk.Image.F3
image=itk.Image[itk.F,3]
# And a normal band image (declared in the module LevelSets)
image=itk.Image.NBNIF33
image=itk.Image[itk.NormalBandNode.IF3,3]
# The function __dir__ is overridden to load modules that
# were not loaded by the lazy loading system
itk.Image.__dir__()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

from __future__ import print_function

import sys
import re
import itk
import os
from optparse import OptionParser

parser = OptionParser(usage='wrappingCoverage.py paths')

parser.add_option(
    "-b",
    "--base",
    dest="base",
    default="Filter",
    help="Base string used to search for the classes (default: Filter).")
parser.add_option(
    "-e",
    "--exclude",
    dest="exclude",
    default=None,
    help="Path of a file with one class to exclude per line (default: None).")
parser.add_option(
    "-E",
    "--no-error",
    action="store_true",
    dest="noError",
    help="Don't generate an error code if all the classes are not wrapped.")

opts, args = parser.parse_args()

# declares classes which will not be wrapped
excluded = set([])
if opts.exclude:
    with open(opts.exclude, 'r') as fp:
        to_exclude = [c.strip() for c in fp.readlines()]
        excluded.update(set(to_exclude))

# get classes from sources
headers = []
for d in args:
    headers += sum([f for p, d, f in os.walk(d)
                   if "Deprecated" not in p and "TestKernel" not in p], [])
classes = set([f[len('itk'):-len('.h')] for f in headers if f.startswith("itk")
              and not f.startswith("itkv3") and
              f.endswith(opts.base + ".h")]) - excluded

# get filter from wrapper files
# remove classes which are not in the toolkit (external projects,
# PyImageFilter, ...)
wrapped = set([a for a in dir(itk) if a.endswith(opts.base)]
              ).intersection(classes)

nonWrapped = classes - wrapped


# print non wrapped classes without much text to stdout, so they can be
# easily reused
for f in sorted(nonWrapped):
    print(f)

# and print stats in stderr to avoid poluting the list above
print("", file=sys.stderr)
print('%i %s' % (len(classes), opts.base), file=sys.stderr)
print('%i wrapped %s' % (len(wrapped), opts.base), file=sys.stderr)
print('%i non wrapped %s' % (len(nonWrapped), opts.base), file=sys.stderr)
covered = len(wrapped) / float(len(classes))
print('%f%% covered' % (covered * 100), file=sys.stderr)
print("", file=sys.stderr)

if not opts.noError:
    sys.exit(len(nonWrapped))



#!/usr/bin/env python

from __future__ import print_function

description = """
Upload all the ExternalData files to Azure Blob storage.

The files corresponding to all the ExternalData content links
in the source tree are archived on the Azure storage container
using the local ExternalData object store.

Requires the azure package:

  https://github.com/Azure/azure-sdk-for-python

There should also be a "md5" container on the storage account.  See:

  http://azure.microsoft.com/en-us/documentation/articles/storage-python-how-to-use-blob-storage/
"""

import argparse
import fnmatch
import os
import sys

from azure.storage import BlobService

def upload_to_azure(content_link, externaldata_object_store,
                    blob_service, current_blobs):
    # get the MD5 checksum
    print('Uploading ' + content_link + ' ...')
    with open(content_link, 'r') as fp:
        md5hash = fp.readline().strip()
    print('Checksum: ' + md5hash)

    if not md5hash in current_blobs:
        object_store = os.path.join(externaldata_object_store, 'MD5', md5hash)
        if not os.path.exists(object_store):
            sys.stderr.write('Could not find the expected object store.\n')
            sys.exit(1)

        blob_service.put_block_blob_from_path('md5', md5hash, object_store)


def run(itk_source_dir, externaldata_object_store, account_name, account_key):
    blob_service = BlobService(account_name=account_name,
                               account_key=account_key)
    blobs = blob_service.list_blobs('md5')
    current_blobs = [blob.name for blob in blobs]

    md5files = []
    for root, dirnames, filenames in os.walk(itk_source_dir):
        for filename in fnmatch.filter(filenames, '*.md5'):
            md5files.append(os.path.join(root, filename))

    for content_link in md5files:
        upload_to_azure(content_link, externaldata_object_store,
                        blob_service, current_blobs)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('--account-name', '-a', default="itkexternaldata",
            help="Azure Storage account name.")
    parser.add_argument('account_key', type=argparse.FileType('r'),
            help="A file that contains the Azure Storage access key.")
    parser.add_argument('itk_source_dir',
            help='Path to the ITK source tree.')
    parser.add_argument('externaldata_object_store',
            help='Path to the ExternalData object store, e.g. ' \
            + 'ExternalData/Objects/ in a build tree.')
    args = parser.parse_args()

    account_key = args.account_key.readline()
    account_key = account_key.strip()

    run(args.itk_source_dir, args.externaldata_object_store,
        account_name=args.account_name, account_key=account_key)



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Test basic properties of modified times
#

import itk

#image and transform are in the same module, but filters are in a different module.
imType = itk.Image[itk.F, 2]
imTypeB = itk.Image[itk.UC, 2]
im = imType.New()

transType = itk.Transform[itk.D, 3]
trans = transType.New()

filtType = itk.AndImageFilter[imTypeB, imTypeB, imTypeB]
filt = filtType.New()

metricType = itk.ImageToImageMetricv4[imType, imType]
met = metricType.New()
#We modify them in the order image, transform, filter
for _ in range(3000):
    im.Modified()
trans.Modified()
met.Modified()
filt.Modified()

#and their Modified times should respect that order.
assert im.GetMTime() < trans.GetMTime()
assert trans.GetMTime() < met.GetMTime()
assert met.GetMTime() < filt.GetMTime()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ThresholdSegmentationLevelSetImageFilterWhiteMatter.png}
#     60 116 5 150 180

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ThresholdSegmentationLevelSetImageFilterVentricle.png}
#     81 112 5 210 250

#     INPUTS: {BrainProtonDensitySlice.png}
#     OUTPUTS: {ThresholdSegmentationLevelSetImageFilterGrayMatter.png}
#     107 69 5 180  210

from __future__ import print_function

import itk
from sys import argv, stderr, exit
import os
itk.auto_progress(2)

# itk.auto_progress(1)

if len(argv) < 8:
    print((
        "Missing Parameters \n Usage: "
        "ThresholdSegmentationLevelSetImageFilter.py inputImage outputImage "
        "seedX seedY InitialDistance LowerThreshold UpperThreshold "
        "[CurvatureScaling == 1.0]"), file=stderr)
    exit(1)

InternalPixelType = itk.F
Dimension = 2
InternalImageType = itk.Image[InternalPixelType, Dimension]

OutputPixelType = itk.UC
OutputImageType = itk.Image[OutputPixelType, Dimension]

thresholder = itk.BinaryThresholdImageFilter[
    InternalImageType, OutputImageType].New()

thresholder.SetLowerThreshold(-1000.0)
thresholder.SetUpperThreshold(0.0)

thresholder.SetOutsideValue(0)
thresholder.SetInsideValue(255)

ReaderType = itk.ImageFileReader[InternalImageType]
WriterType = itk.ImageFileWriter[OutputImageType]

reader = ReaderType.New()
writer = WriterType.New()

reader.SetFileName(argv[1])
writer.SetFileName(argv[2])


FastMarchingFilterType = itk.FastMarchingImageFilter[
    InternalImageType,
    InternalImageType]
fastMarching = FastMarchingFilterType.New()

ThresholdSegLvlSetImgFilterType = itk.ThresholdSegmentationLevelSetImageFilter[
    InternalImageType,
    InternalImageType,
    InternalPixelType]
thresholdSegmentation = ThresholdSegLvlSetImgFilterType.New()
thresholdSegmentation.SetPropagationScaling(1.0)
if len(argv) > 8:
    thresholdSegmentation.SetCurvatureScaling(float(argv[8]))
else:
    thresholdSegmentation.SetCurvatureScaling(1.0)

thresholdSegmentation.SetMaximumRMSError(0.02)
thresholdSegmentation.SetNumberOfIterations(1200)

thresholdSegmentation.SetUpperThreshold(float(argv[7]))
thresholdSegmentation.SetLowerThreshold(float(argv[6]))
thresholdSegmentation.SetIsoSurfaceValue(0.0)

thresholdSegmentation.SetInput(fastMarching.GetOutput())
thresholdSegmentation.SetFeatureImage(reader.GetOutput())
thresholder.SetInput(thresholdSegmentation.GetOutput())
writer.SetInput(thresholder.GetOutput())

NodeType = itk.LevelSetNode[InternalPixelType, Dimension]
NodeContainer = itk.VectorContainer[itk.UI, NodeType]
seeds = NodeContainer.New()
seedPosition = [int(argv[3]), int(argv[4])]

initialDistance = float(argv[5])

node = NodeType()

seedValue = - initialDistance

node.SetValue(seedValue)
node.SetIndex(seedPosition)

seeds.Initialize()
seeds.InsertElement(0, node)

fastMarching.SetTrialPoints(seeds)

fastMarching.SetSpeedConstant(1.0)


reader.Update()
fastMarching.SetOutputSize(
    reader.GetOutput().GetBufferedRegion().GetSize())
writer.Update()

itk.echo(thresholdSegmentation)


InternalWriterType = itk.ImageFileWriter[InternalImageType]

outputDirectory = os.path.dirname(argv[2])
mapWriter = InternalWriterType.New()
mapWriter.SetInput(fastMarching.GetOutput())
mapWriter.SetFileName(os.path.join(outputDirectory, "fastMarchingImage.mha"))
mapWriter.Update()

speedWriter = InternalWriterType.New()
speedWriter.SetInput(thresholdSegmentation.GetSpeedImage())
speedWriter.SetFileName(os.path.join(outputDirectory, "speedTermImage.mha"))
speedWriter.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the BinaryDilateImageFilter
#

from InsightToolkit import *
from sys import argv

reader = itkImageFileReaderIUC2.New()
reader.SetFileName(argv[1])
kernel = itkFlatStructuringElement2.Ball(5)
filter = itkBinaryDilateImageFilterIUC2IUC2SE2.New()
filter.SetInput(reader.GetOutput())
filter.SetDilateValue(200)
filter.SetKernel(kernel)
writer = itkImageFileWriterIUC2.New()
writer.SetInput(filter.GetOutput())
writer.SetFileName(argv[2])

writer.Update()



#==========================================================================
#
#   Copyright Insight Software Consortium
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0.txt
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#==========================================================================*/

#
#  Example on the use of the MeanImageFilter
#

import itk
from sys import argv
import os
itk.auto_progress(2)

dim = 2
PixelType = itk.F
ImageType = itk.Image[PixelType, dim]
ComplexImageType = itk.Image[itk.complex[PixelType], dim]

reader = itk.ImageFileReader[ImageType].New(FileName=argv[1])
fftFilter = itk.ForwardFFTImageFilter[ImageType, ComplexImageType].New(reader)

# why this Update() ?
fftFilter.Update()

complexWriter = itk.ImageFileWriter[ComplexImageType].New(
    fftFilter,
    FileName=os.path.join(os.path.dirname(argv[2]), "complexImage.mhd"))
complexWriter.Update()

realFilter = itk.ComplexToRealImageFilter[
    ComplexImageType, ImageType].New(fftFilter)

WritePixelType = itk.UC
WriteImageType = itk.Image[WritePixelType, dim]
intensityRescaler = itk.RescaleIntensityImageFilter[
    ImageType,
    WriteImageType].New(
    realFilter,
    OutputMinimum=0,
    OutputMaximum=255)
writer = itk.ImageFileWriter[WriteImageType].New(
    intensityRescaler,
    FileName=argv[2])
writer.Update()

imaginaryFilter = itk.ComplexToImaginaryImageFilter[
    ComplexImageType, ImageType].New(fftFilter)
intensityRescaler.SetInput(imaginaryFilter.GetOutput())
writer.SetFileName(argv[3])
writer.Update()
